@@ -26,7 +26,6 @@
 
 from __future__ import absolute_import, print_function
 
-import apt
 import apt_pkg
 import datetime
 import dbus
@@ -44,9 +43,10 @@ import time
 import gi
 gi.require_version("Gdk", "3.0")
 gi.require_version("Gtk", "3.0")
+gi.require_version("Handy", "1")
 gi.require_version("PackageKitGlib", "1.0")
 from gi.repository import PackageKitGlib as packagekit
-from gi.repository import GObject, Gdk, Gtk, Gio, GLib
+from gi.repository import GObject, Gdk, Gtk, Gio, GLib, Handy
 
 from .SimpleGtkbuilderApp import SimpleGtkbuilderApp
 from .DialogAdd import DialogAdd
@@ -54,7 +54,7 @@ from .DialogMirror import DialogMirror
 from .DialogEdit import DialogEdit
 from .DialogCacheOutdated import DialogCacheOutdated
 from .DialogAddSourcesList import DialogAddSourcesList
-from .LivepatchPage import LivepatchPage
+from .UbuntuProPage import UbuntuProPage
 
 import softwareproperties
 import softwareproperties.distro
@@ -162,6 +162,11 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         SoftwareProperties.__init__(self, options=options, datadir=datadir)
         Gtk.Window.set_default_icon_name("software-properties")
 
+        # Support GNOME 42 dark mode
+        Handy.init()
+        style_manager = Handy.StyleManager.get_default()
+        style_manager.set_color_scheme(Handy.ColorScheme.PREFER_LIGHT)
+
         SimpleGtkbuilderApp.__init__(self,
                                      os.path.join(datadir, "gtkbuilder", "main.ui"),
                                      domain="software-properties")
@@ -224,7 +229,9 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         # used to store the handlers of callbacks
         self.handlers = {}
 
-        self.apt_cache = {}
+        # Initialise and store the apt cache
+        self.init_apt_cache()
+
         self.pk_task = None
 
         # Put some life into the user interface:
@@ -243,8 +250,8 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.show_distro()
         # Setup and show the Additonal Drivers tab
         self.init_drivers()
-        # Setup and show the LivePatch tab
-        self.init_livepatch()
+        # Setup and show the Ubuntu Pro tab
+        self.init_ubuntu_pro()
 
         # Connect to switch-page before setting initial tab. Otherwise the
         # first switch goes unnoticed.
@@ -286,6 +293,17 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.handlers[self.combobox_release_upgrades] = \
              self.combobox_release_upgrades.connect("changed", 
                                                     self.on_combobox_release_upgrades_changed)
+    def init_apt_cache(self):
+        apt_pkg.init_config()
+        apt_pkg.init_system()
+        self.apt_cache = apt_pkg.Cache(None)
+        self.depcache = apt_pkg.DepCache(self.apt_cache)
+        self.records = apt_pkg.PackageRecords(self.apt_cache)
+
+    def update_apt_cache(self):
+        self.apt_cache = apt_pkg.Cache(None)
+        self.depcache = apt_pkg.DepCache(self.apt_cache)
+        self.records = apt_pkg.PackageRecords(self.apt_cache)
 
     def init_auto_update(self):
         """ Set up the widgets that allow to configure the update automation """
@@ -979,7 +997,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
     # dbus events
     def on_config_modified(self):
         """The config was changed and now needs to be saved and reloaded"""
-        apt.apt_pkg.init_config()
+        apt_pkg.init_config()
         self.button_revert.set_sensitive(True)
 
     def on_keys_modified(self):
@@ -1179,7 +1197,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
     def on_delete_event(self, widget, args):
         """Close the window if requested"""
         self.on_close_button(widget)
-        return self.quit_when_livepatch_responds
+        return False
 
     def on_close_button(self, widget):
         """Show a dialog that a reload of the channel information is required
@@ -1190,12 +1208,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
                                     self.datadir)
             d.run()
 
-        self.quit_when_livepatch_responds = False
-        if self.livepatch_page.waiting_livepatch_response:
-            self.quit_when_livepatch_responds = True
-            self.hide()
-        else:
-            self.quit()
+        self.quit()
 
     def on_button_add_cdrom_clicked(self, widget):
         """ when a cdrom is requested for adding """
@@ -1227,7 +1240,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         if not installs_pending:
             self.progress_bar.set_visible(False)
             self.clear_changes()
-            self.apt_cache = apt.Cache()
+            self.update_apt_cache()
             self.set_driver_action_status()
             self.update_label_and_icons_from_status()
             self.button_driver_revert.set_visible(True)
@@ -1240,19 +1253,20 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         installs = []
         removals = []
 
+        has_nvidia = False
         for pkg in self.driver_changes:
-            if pkg.is_installed:
-                removals.append(self.get_package_id(pkg.installed))
+            if pkg.current_ver:
+                removals.append(self.get_package_id(self.apt_cache, pkg))
                 # The main NVIDIA package is only a metapackage.
                 # We need to collect its dependencies, so that
                 # we can uninstall the driver properly.
-                if 'nvidia' in pkg.shortname:
-                    for dep in self.get_dependencies(self.apt_cache, pkg.shortname, 'nvidia'):
-                        dep_pkg = self.apt_cache[dep]
-                        if dep_pkg.is_installed:
-                            removals.append(self.get_package_id(dep_pkg.installed))
+                if 'nvidia' in pkg.name:
+                    has_nvidia = True
+                    for dep in self.get_dependencies(self.apt_cache, pkg, 'nvidia'):
+                        if dep.current_ver:
+                            removals.append(self.get_package_id(self.apt_cache, dep))
             else:
-                installs.append(self.get_package_id(pkg.candidate))
+                installs.append(self.get_package_id(self.apt_cache, pkg))
 
         self.cancellable = Gio.Cancellable()
         try:
@@ -1270,6 +1284,11 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
                             installs_pending  # callback data
                  )
             if installs:
+                if has_nvidia:
+                    to_install = []
+                    for item in installs:
+                        name = item.split(';')[0]
+                        to_install.append(name)
                 self.pk_task.install_packages_async(installs,
                         self.cancellable,  # cancellable
                         self.on_driver_changes_progress,
@@ -1377,7 +1396,6 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         # WARNING: This is run in a separate thread.
         self.detect_called = True
         try:
-            self.apt_cache = apt.Cache()
             self.devices = detect.system_device_drivers(self.apt_cache)
         except:
             # Catch all exceptions and feed them to apport.
@@ -1396,22 +1414,16 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             return
 
         pkg = None
-        try:
-            if pkg_name:
-                pkg = self.apt_cache[pkg_name]
-                # If the package depends on dkms
-                # we need to install the correct linux metapackage
-                # so that users get the latest headers
-                if 'dkms' in pkg.candidate.record['Depends']:
-                    linux_meta = detect.get_linux(self.apt_cache)
-                    if (linux_meta and
-                        linux_meta not in self.driver_changes):
-                        # Install the linux metapackage
-                        lmp = self.apt_cache[linux_meta]
-                        if not lmp.is_installed:
-                            self.driver_changes.append(lmp)
-        except (AttributeError, KeyError):
-            pass
+        if pkg_name:
+            pkg = self.apt_cache[pkg_name]
+            # Add the matching linux modules package when available
+            try:
+                modules_package = detect.get_linux_modules_metapackage(self.apt_cache, pkg_name)
+                modules_package_obj = self.apt_cache[modules_package]
+                if modules_package and not modules_package_obj.current_ver:
+                    self.driver_changes.append(modules_package_obj)
+            except KeyError:
+                pass
 
         if button.get_active():
             if pkg in self.driver_changes:
@@ -1431,7 +1443,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
 
             if (pkg is not None
                 and pkg not in self.driver_changes
-                    and pkg.is_installed):
+                    and pkg.current_ver):
                 self.driver_changes.append(pkg)
 
         self.button_driver_revert.set_sensitive(bool(self.driver_changes))
@@ -1493,11 +1505,14 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
 
             try:
                 pkg = self.apt_cache[pkg_driver_name]
-                installed = pkg.is_installed
-                if pkg.candidate is not None:
-                    description = _("Using {} from {}").format(pkg.candidate.summary, pkg.shortname)
+                installed = pkg.current_ver
+                candidate = self.depcache.get_candidate_ver(pkg)
+
+                if candidate is not None:
+                    self.records.lookup(candidate.file_list[0])
+                    description = _("Using {} from {}").format(self.records.short_desc, pkg_driver_name)
                 else:
-                    description = _("Using {}").format(pkg.shortname)
+                    description = _("Using {}").format(pkg_driver_name)
             except KeyError:
                 print("WARNING: a driver ({}) doesn't have any available package associated: {}".format(pkg_driver_name, current_driver))
                 continue
@@ -1627,7 +1642,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         for device in self.devices:
             for pkg_name in self.devices[device]['drivers']:
                 pkg = self.apt_cache[pkg_name]
-                if not self.devices[device]['drivers'][pkg_name]['free'] and pkg.is_installed:
+                if not self.devices[device]['drivers'][pkg_name]['free'] and pkg.current_ver:
                     self.nonfree_drivers = self.nonfree_drivers + 1
 
         if self.nonfree_drivers > 0:
@@ -1639,6 +1654,5 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         else:
             self.label_driver_action.set_label(_("No proprietary drivers are in use."))
 
-    def init_livepatch(self):
-        self.livepatch_page = LivepatchPage(self)
-
+    def init_ubuntu_pro(self):
+        self.ubuntu_pro_page = UbuntuProPage(self)
