@@ -55,7 +55,6 @@ from .CdromProgress import CdromProgress
 
 from UbuntuDrivers import detect
 import sys
-import apt
 from functools import partial
 import aptsources.distro
 import logging
@@ -345,6 +344,18 @@ class SoftwarePropertiesQt(SoftwareProperties):
     self.userinterface.combobox_server.addItem(_("Other..."))
     self.other_mirrors_index = self.userinterface.combobox_server.count() - 1
 
+  def init_apt_cache(self):
+    apt_pkg.init_config()
+    apt_pkg.init_system()
+    self.apt_cache = apt_pkg.Cache(None)
+    self.depcache = apt_pkg.DepCache(self.apt_cache)
+    self.records = apt_pkg.PackageRecords(self.apt_cache)
+
+  def update_apt_cache(self):
+    self.apt_cache = apt_pkg.Cache(None)
+    self.depcache = apt_pkg.DepCache(self.apt_cache)
+    self.records = apt_pkg.PackageRecords(self.apt_cache)
+
   def show_distro(self):
     """
     Represent the distro information in the user interface
@@ -736,7 +747,7 @@ class SoftwarePropertiesQt(SoftwareProperties):
 
   def on_restore_clicked(self):
     """Restore the original keys"""
-    self.apt_key.update()
+    self.update_apt_cache()
     self.show_keys()
 
   def on_pktask_progress(self, progress, ptype, udata=(None,)):
@@ -859,7 +870,7 @@ class SoftwarePropertiesQt(SoftwareProperties):
     if not installs_pending:
       self.progress_bar.setVisible(False)
       self.clear_changes()
-      self.apt_cache = apt.Cache()
+      self.update_apt_cache()
       self.set_driver_action_status()
       self.update_label_and_icons_from_status()
       self.button_driver_revert.setVisible(True)
@@ -885,19 +896,20 @@ class SoftwarePropertiesQt(SoftwareProperties):
     installs = []
     removals = []
 
+    has_nvidia = False
     for pkg in self.driver_changes:
-      if pkg.is_installed:
-          removals.append(self.get_package_id(pkg.installed))
+      if pkg.current_ver:
+          removals.append(self.get_package_id(self.apt_cache, pkg))
           # The main NVIDIA package is only a metapackage.
           # We need to collect its dependencies, so that
           # we can uninstall the driver properly.
-          if 'nvidia' in pkg.shortname:
-            for dep in get_dependencies(self.apt_cache, pkg.shortname, 'nvidia'):
-              dep_pkg = self.apt_cache[dep]
-              if dep_pkg.is_installed:
-                removals.append(self.get_package_id(dep_pkg.installed))
+          if 'nvidia' in pkg.name:
+            has_nvidia = True
+            for dep in self.get_dependencies(self.apt_cache, pkg, 'nvidia'):
+              if dep.current_ver:
+                removals.append(self.get_package_id(self.apt_cache, dep))
       else:
-        installs.append(self.get_package_id(pkg.candidate))
+        installs.append(self.get_package_id(self.apt_cache, pkg))
 
     self.cancellable = Gio.Cancellable()
     try:
@@ -915,6 +927,11 @@ class SoftwarePropertiesQt(SoftwareProperties):
                   installs_pending  # callback data
          )
       if installs:
+        if has_nvidia:
+          to_install = []
+          for item in installs:
+            name = item.split(';')[0]
+            to_install.append(name)
         self.pk_task.install_packages_async(installs,
               self.cancellable,  # cancellable
               self.on_driver_changes_progress,
@@ -1019,8 +1036,8 @@ class SoftwarePropertiesQt(SoftwareProperties):
   def detect_drivers(self):
     # WARNING: This is run in a separate thread.
     self.detect_called = True
+    self.init_apt_cache()
     try:
-      self.apt_cache = apt.Cache()
       self.devices = detect.system_device_drivers(self.apt_cache)
     except:
       # Catch all exceptions and feed them to apport.
@@ -1039,21 +1056,17 @@ class SoftwarePropertiesQt(SoftwareProperties):
       return
 
     pkg = None
-    try:
-      if pkg_name:
-        pkg = self.apt_cache[pkg_name]
-        # If the package depends on dkms
-        # we need to install the correct linux metapackage
-        # so that users get the latest headers
-        if 'dkms' in pkg.candidate.record['Depends']:
-          linux_meta = detect.get_linux(self.apt_cache)
-          if (linux_meta and linux_meta not in self.driver_changes):
-            # Install the linux metapackage
-            lmp = self.apt_cache[linux_meta]
-            if not lmp.is_installed:
-              self.driver_changes.append(lmp)
-    except KeyError:
-      pass
+
+    if pkg_name:
+      pkg = self.apt_cache[pkg_name]
+      # Add the matching linux modules package when available
+      try:
+        modules_package = detect.get_linux_modules_metapackage(self.apt_cache, pkg_name)
+        modules_package_obj = self.apt_cache[modules_package]
+        if modules_package and not modules_package_obj.current_ver:
+          self.driver_changes.append(modules_package_obj)
+      except KeyError:
+          pass
 
     if button.isChecked():
       if pkg in self.driver_changes:
@@ -1069,7 +1082,7 @@ class SoftwarePropertiesQt(SoftwareProperties):
       if modalias not in self.orig_selection:
           self.orig_selection[modalias] = button
 
-      if (pkg is not None and pkg not in self.driver_changes and pkg.is_installed):
+      if (pkg is not None and pkg not in self.driver_changes and pkg.current_ver):
           self.driver_changes.append(pkg)
 
     self.button_driver_revert.setEnabled(bool(self.driver_changes))
@@ -1137,11 +1150,13 @@ None if not applicable
 
       try:
         pkg = self.apt_cache[pkg_driver_name]
-        installed = pkg.is_installed
-        if pkg.candidate is not None:
-          description = _("Using {} from {}").format(pkg.candidate.summary, pkg.shortname)
+        installed = pkg.current_ver or False
+        candidate = self.depcache.get_candidate_ver(pkg)
+        if candidate is not None:
+          self.records.lookup(candidate.file_list[0])
+          description = _("Using {} from {}").format(self.records.short_desc, pkg_driver_name)
         else:
-          description = _("Using {}").format(pkg.shortname)
+          description = _("Using {}").format(pkg_driver_name)
       except KeyError:
         print("WARNING: a driver ({}) doesn't have any available package associated: {}".format(pkg_driver_name, current_driver))
         continue
@@ -1277,7 +1292,7 @@ licence=licence)
     for device in self.devices:
       for pkg_name in self.devices[device]['drivers']:
         pkg = self.apt_cache[pkg_name]
-        if not self.devices[device]['drivers'][pkg_name]['free'] and pkg.is_installed:
+        if not self.devices[device]['drivers'][pkg_name]['free'] and pkg.current_ver:
           self.nonfree_drivers = self.nonfree_drivers + 1
 
     if self.nonfree_drivers > 0:
